# DynamicPreview - (c) 2014-2017 Juxtaposition. All Rights Reserved.
# This code cannot be redistributed without permission from juxtaposition.jp
# For more information, consult your DynamicPreview license.
#
package DynamicPreview::CMS;

use strict;
use warnings;
use utf8;
use Data::Dumper;

use constant TRACE => 1;

sub trace {
    my ($msg) = @_;
    print STDERR $msg if TRACE;
    return;
}

# CMS methods
sub show_preview {
    my $app = shift;
    my $param = {};
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';

    # $app->validate_magic or return;

    my $blog_id = $app->param('blog_id');
    my $blog = $app->blog;
    my $id = $app->param('id');
    my $entry_type = $app->param('_type') || 'entry';
    my @tmpl_ids = $app->multi_param('tmpl_id');
    my $cat_id = $app->param('cat_id');
    my $rn = $app->param('r');

    # entry または category があれば良い
    if($blog_id && ($id || $cat_id) && @tmpl_ids) {
        my ($entry, $categories);
        if($id) {
            require MT::Entry;
            $entry = MT::Entry->load({
                id => $id,
                blog_id => $blog_id,
            }) or return $app->errtrans('Invalid request.');
            $categories = $entry->categories;
            unless(@$categories) {
                require MT::Category;
                my $cat = MT::Category->load(
                    {
                        blog_id => $blog_id,
                    },
                    {
                        sort      => 'id',
                        direction => 'ascend',
                    }
                );
                unless ($cat) {
                    $cat = new MT::Category;
                    $cat->label( $app->translate("Preview") );
                    $cat->basename("preview");
                    $cat->parent(0);
                }
                $categories = [ $cat ];
            }
        }
        elsif($cat_id) {
            require MT::Category;
            my $cat = MT::Category->load($cat_id)
                or return $app->errtrans('Invalid request');
            $categories = [ $cat ];
        }

        $param->{can_edit_entry} = $app->user && $entry ? $app->user->can_edit_entry($entry) : 0;

        require MT::Template;
        require MT::TemplateMap;
        my @tmpl = MT::Template->load({
            id => \@tmpl_ids,
        }, {
            sort => 'name',
            direction => 'ascend',
        });
        return $app->errtrans('Invalid request.') unless @tmpl > 0;

        my @items = ();
        foreach my $tmpl (@tmpl) {
            my $item = {
                id => $tmpl->id,
                name => $tmpl->name,
                blog_id => $tmpl->blog_id,
                type => $tmpl->type,
                magic_token => $app->current_magic,
                query => undef,
            };
            my $type = lc $tmpl->type;
            if($entry && $type eq 'individual' || $type eq 'page') {
                my $query = $app->uri_params(
                    mode => 'dynamic_preview_entry',
                    args => {
                        id => $entry->id,
                        blog_id => $entry->blog_id,
                        _type => $entry_type,
                        tmpl_id => $tmpl->id,
                        magic_token => $app->current_magic,
                        ( $rn ? ( r => $rn ) : () ),
                    },
                );
                $query =~ s!^\?!!;
                $item->{query} = $query;
                push @items, $item;
            }
            elsif($type eq 'archive') {
                my $map = MT::TemplateMap->load({
                    template_id => $tmpl->id,
                    build_type => { op => '!=', value => 0 },
                    archive_type => 'Category',
                });
                if($map) {
                    $item->{archive_type} = lc $map->archive_type;
                    foreach (@$categories) {
                        my $new_item = { %$item };
                        $new_item->{cat_id} = $_->id;
                        $new_item->{cat_label} = $_->label;
                        push @items, $new_item;
                    }
                }
                else {
                    push @items, $item;
                }
            }
            elsif($type eq 'index') {
                push @items, $item;
            }
        }
        $param->{entry_type} = $entry ? ($entry->class_type || 'entry') : '';
        $param->{item_loop} = \@items;
        $param->{from_external_link} = 1;
        my $idx = 0;
        my %sort_order = map { $_ => $idx++ } @tmpl_ids;
        $param->{sort_order} = \%sort_order;
    }

    $param->{id} = $id;
    $param->{entry_type} = $entry_type unless $param->{entry_type};
    $param->{cat_id} = $cat_id;
    $param->{blog_id} = $blog_id;
    $param->{tmpl_id} = @tmpl_ids ? \@tmpl_ids : undef;
    $param->{preview_url} = $app->uri;
    $param->{has_extra} = $plugin->has_extra;
    $param->{dirty_entry} = $app->param('dirty');
    $param->{rev_number} = defined($rn) ? $rn : undef;
    my $tmpl = $plugin->load_tmpl('show.tmpl', $param);
    return $tmpl;
}

sub show_preview_cd {
    my $app = shift;
    my $param = {};
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';

    # $app->validate_magic or return;

    my $blog_id = $app->param('blog_id');
    my $blog = $app->blog;
    my $id = $app->param('id');
    my $obj_type = $app->param('_type') || 'content_data';
    my @tmpl_ids = $app->multi_param('tmpl_id');
    my $rn = $app->param('r');
    my $content_type_id = $app->param('content_type_id');

    if($blog_id && $id && @tmpl_ids) {
        require MT::ContentData;
        my $content_data = MT::ContentData->load({
            id => $id,
            blog_id => $blog_id,
        }) or return $app->errtrans('Invalid request.');

        $param->{can_edit_content_data} = $app->user && $content_data 
            ? $app->user->permissions($blog_id)->can_edit_content_data($content_data, $app->user)
            : 0;

        require MT::Template;
        require MT::TemplateMap;
        my @tmpl = MT::Template->load({
            id => \@tmpl_ids,
        }, {
            sort => 'name',
            direction => 'ascend',
        });
        return $app->errtrans('Invalid request.') unless @tmpl > 0;

        my @items = ();
        foreach my $tmpl (@tmpl) {
            my $item = {
                id => $tmpl->id,
                name => $tmpl->name,
                blog_id => $tmpl->blog_id,
                type => $tmpl->type,
                magic_token => $app->current_magic,
                query => undef,
            };
            my $type = lc $tmpl->type;
            if($content_data && $type eq 'ct') {
                my $query = $app->uri_params(
                    mode => 'dynamic_preview_content_data',
                    args => {
                        id => $content_data->id,
                        content_type_id => $content_data->content_type_id,
                        blog_id => $content_data->blog_id,
                        _type => $obj_type,
                        tmpl_id => $tmpl->id,
                        magic_token => $app->current_magic,
                        from_external_link => 1,
                        ( $rn ? ( r => $rn ) : () ),
                    },
                );
                $query =~ s!^\?!!;
                $item->{query} = $query;
                push @items, $item;
                
                $content_type_id = $content_data->content_type_id;
            }
=pod
            elsif($type eq 'archive') {
                my $map = MT::TemplateMap->load({
                    template_id => $tmpl->id,
                    build_type => { op => '!=', value => 0 },
                    archive_type => 'Category',
                });
                if($map) {
                    $item->{archive_type} = lc $map->archive_type;
                    foreach (@$categories) {
                        my $new_item = { %$item };
                        $new_item->{cat_id} = $_->id;
                        $new_item->{cat_label} = $_->label;
                        push @items, $new_item;
                    }
                }
                else {
                    push @items, $item;
                }
            }
=cut
            elsif($type eq 'index') {
                push @items, $item;
            }
        }
        $param->{obj_type} = 'content_data';
        $param->{item_loop} = \@items;
        $param->{from_external_link} = 1;
        my $idx = 0;
        my %sort_order = map { $_ => $idx++ } @tmpl_ids;
        $param->{sort_order} = \%sort_order;
    }

    $param->{id} = $id;
    $param->{obj_type} = $obj_type unless $param->{obj_type};
    $param->{blog_id} = $blog_id;
    $param->{content_type_id} = $content_type_id;
    $param->{tmpl_id} = @tmpl_ids ? \@tmpl_ids : undef;
    $param->{preview_url} = $app->uri;
    $param->{has_extra} = $plugin->has_extra;
    $param->{dirty_obj} = $app->param('dirty');
    $param->{rev_number} = defined($rn) ? $rn : undef;
    my $tmpl = $plugin->load_tmpl('show_cd.tmpl', $param);
    return $tmpl;
}

sub _load_users {
    my $app = shift;
    my ($entry, $category, $content_data) = @_;
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';
    return (wantarray ? () : undef) unless $entry || $category || $content_data;

    my $blog_id = $app->param('blog_id');

    require MT::Author;
    my @authors;
    my $iter = MT::Author->load_iter();
    while(my $author = $iter->()) {
        my $perms = $author->permissions($blog_id)
            or next;
        if($author->is_superuser
            || $author->can_do('preview', at_least_one => 1, blog_id => $blog_id)
            || ($entry && $author->can_edit_entry($entry))
            || ($category && $author->can_edit_templates())
            || ($content_data && $perms->can_edit_content_data($content_data, $author))) {
            push @authors, $author;
        }
    }
    return wantarray ? @authors : $authors[0];
}

sub edit_mail {
    my $app = shift;
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';
    return $app->error(
        $app->translate('No permissions.')
    ) unless $plugin->has_extra;

    my $blog_id = $app->param('blog_id');
    my $blog = $app->blog;
    my $preview_url = $app->param('preview_url');
    my $type = $app->param('_type') || 'entry';
    my $entry_id = $app->param('entry_id');
    my $cat_id = $app->param('cat_id');
    my $content_data_id = $app->param('content_data_id');
    my $content_type_id = undef;
    if($type eq 'entry' && !$entry_id && !$cat_id) {
        return $app->error( $app->translate('Entry ID or category ID was not provided') );
    }
    elsif($type eq 'content_data' && !$content_data_id) {
        return $app->error($app->translate('Content Data ID was not provided'));
    }

    my ($entry, $category, $content_data, $content_type);

    if($entry_id) {
        require MT::Entry;
        $entry = MT::Entry->load($entry_id)
            or return $app->error(
                $app->translate("No such entry '[_1]'", $entry_id)
            );
    }
    if($cat_id) {
        require MT::Category;
        $category = MT::Category->load($cat_id)
            or return $app->error(
                $app->translate("No such category '[_1]'", $cat_id)
            );
    }
    if($content_data_id) {
        require MT::ContentData;
        $content_data = MT::ContentData->load($content_data_id)
            or return $app->error(
                $app->translate("No such content data '[_1]'", $content_data_id)
            );
        $content_type = $content_data->content_type;
        $content_type_id = $content_type->id;
    }

    my @users = _load_users($app, $entry, $category, $content_data);

    my $subject;
    my $subj_tmpl_str = $plugin->dynamic_preview_mail_subject($blog_id);
    if($subj_tmpl_str) {
        my $subj_tmpl = MT::Template->new(type => 'scalarref', source => \$subj_tmpl_str);
        $app->set_default_tmpl_params($subj_tmpl);
        my $ctx = $subj_tmpl->context;
        $ctx->stash('blog', $blog);
        my $entries = [];
        push @$entries, $entry if $entry;
        $ctx->stash('entries', $entries);
        $ctx->stash('entry', $entry);
        $ctx->stash('category', $category);
        $ctx->stash('content', $content_data);
        $ctx->stash('content_type', $content_type);
        $subject = $subj_tmpl->output({
            blog => $blog,
            entry => $entry,
            category => $category,
            content => $content_data,
            content_type => $content_type,
            type => $type,
        });
        if(!$subject && $subj_tmpl->errstr) {
            return $app->error($subj_tmpl->errstr);
        }
    }
    unless($subject) {
        my $label = $type eq 'entry' 
            ? $entry ? $entry->title : $category ? $category->label : ''
            : $content_data->label;
        $subject = $app->translate(
            "([_1]) Request for check - [_2]", $blog->name, $label
        );
    }

    my $param = {
        entry_id => $entry_id,
        preview_url => $preview_url,
        users => \@users,
        subject => $subject,
        cat_id => $cat_id,
        content_data_id => $content_data_id,
        content_type_id => $content_type_id,
        type => $type,
    };
    my $tmpl = $plugin->load_tmpl('edit.tmpl', $param);
    return $tmpl;
}

use MT::Util qw( is_valid_email is_url dirify );
use MT::I18N qw( wrap_text );

sub send_mail {
    my $app = shift;
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';
    return $app->error(
        $app->translate('No permissions.')
    ) unless $plugin->has_extra;
    $app->validate_magic or return;

    my $type = $app->param('_type') || 'entry';
    my $entry_id = $app->param('entry_id');
    my $cat_id = $app->param('cat_id');
    my $content_data_id = $app->param('content_data_id');
    my $content_type_id = undef;
    if($type eq 'entry' && !$entry_id && !$cat_id) {
        return $app->error( $app->translate('Entry ID or category ID was not provided') );
    }
    elsif($type eq 'content_data' && !$content_data_id) {
        return $app->error($app->translate('Content Data ID was not provided'));
    }

    my ($entry, $category, $blog, $content_data, $content_type);

    if($entry_id) {
        require MT::Entry;
        $entry = MT::Entry->load($entry_id)
            or return $app->error(
                $app->translate("No such entry '[_1]'", $entry_id)
            );
        $blog = $entry->blog;
    }
    if($cat_id) {
        require MT::Category;
        $category = MT::Category->load($cat_id)
            or return $app->error(
                $app->translate("No such category '[_1]'", $cat_id)
            );
        $blog = MT::Blog->load($category->blog_id);
    }
    if($content_data_id) {
        require MT::ContentData;
        $content_data = MT::ContentData->load($content_data_id)
            or return $app->error(
                $app->translate("No such content data '[_1]'", $content_data_id)
            );
        $blog = $content_data->blog;
        $content_type = $content_data->content_type;
        $content_type_id = $content_type->id;
    }

    my $user = $app->user;
    my $saved_app_blog = $app->blog;
    $app->blog($blog);

    # my $author = $entry ? $entry->author : undef;
    my $author = $entry ? $entry->author 
                        : $content_data ? $content_data->author 
                                        : $user;
    my $preview_url = $app->param('preview_url')
        or return $app->error(
            $app->translate('No preview URL was provided')
        );

    my $entries = [];
    push @$entries, $entry if $entry;
    my $params = {
        blog => $blog,
        entry => $entry,
        entries => $entries,
        category => $category,
        # entry_author => $author ? 1 : 0,
        entry_author => $entry ? 1 : 0,
        author => $author,
        has_author => $author ? 1 : 0,
        preview_url => $preview_url,
        content => $content_data,
        content_type => $content_type,
        type => $type,
    };

    my $cols = 72;
    my $message = $app->param('message');
    $params->{message} = wrap_text( $message, $cols, '', '');
    
    my $addrs;
    if($app->param('send_notify_emails')) {
        # my @addrs = split(/[\n\r,]+/, $app->param('send_notify_emails'));
        my @addrs = $app->multi_param('send_notify_emails');
        foreach my $addr (@addrs) {
            next unless is_valid_email($addr);
            $addrs->{$addr} = 1;
        }
    }

    keys %$addrs
        or return $app->error(
            $app->translate('No valid recipients were found for the entry notification.')
        );

    my $address;
    if($author) {
        $address = defined $author->nickname
            ? $author->nickname . ' <' . $author->email . '>'
            : $author->email;
    }
    else {
        $address = $app->config('EmailAddressMain');
        $params->{from_address} = $address;
    }

    my $tmpl_str = $plugin->dynamic_preview_mail_template($saved_app_blog->id)
        or return $app->error(
            $app->translate('No email template.')
        );
    my $body = build_mail_body($app, $tmpl_str, $params)
        or return;

    my $subject = $app->param('subject')
        or return $app->error(
            $app->translate('You must enter the subject.')
        );
    if($app->current_language ne 'ja') {
        $subject =~ s![\x80-\xFF]!!g;
    }
    my $header = {
        id => 'nofity_preview',
        To => $address,
        From => $address,
        Subject => $subject,
    };
    my $charset = $app->config('MailEncoding') || $app->charset;
    $header->{'Content-Type'} = qq!text/plain; charset="$charset"!;
    my $i = 1;
    require MT::Mail;
    unless(exists $params->{from_address}) {
        MT::Mail->send($header, $body)
            or return $app->errtrans(
                "Error sending mail ([_1]): Try another MailTransfer setting?",
                MT::Mail->errstr
            );
    }
    delete $header->{To};

    my @email_to_send;
    my @addresses_to_send = grep $_, keys %$addrs;
    if($app->config('EmailNotificationBcc')) {
        while(@addresses_to_send) {
            push @email_to_send, {
                %$header,
                Bcc => [ splice(@addresses_to_send, 0, 20) ], 
            }
        }
    }
    else {
        @email_to_send = map {
            { %$header, To => $_ }
        } @addresses_to_send;
    }

    foreach my $info (@email_to_send) {
        MT::Mail->send($info, $body)
            or return $app->errtrans(
                "Error sending mail ([_1]): Try another MailTransfer setting?",
                MT::Mail->errstr
            );
    }

    $app->blog($saved_app_blog);
    my $tmpl = $plugin->load_tmpl('send.tmpl', {});
    return $tmpl;
}

sub build_mail_body {
    my ($app, $tmpl_str, $params) = @_;
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';
    my $tmpl = MT::Template->new(type => 'scalarref', source => \$tmpl_str);
    $app->set_default_tmpl_params($tmpl);
    my $blog = $params->{blog} || undef;
    my $ctx = $tmpl->context;
    $ctx->stash('blog_id', $blog->id) if $blog;
    foreach my $name (qw/blog entry author category entries content content_type/) {
        $ctx->stash($name, delete $params->{$name}) if $params->{$name};
    }
    return $app->build_page_in_mem($tmpl, $params);
}

sub _setup_category_ids_param {
    my ($app) = @_;

    my $type = $app->param('_type') || 'entry';
    my $entry_class = $app->model($type);
    my $blog_id = $app->param('blog_id');
    my $id = $app->param('id')
        or return $app->errtrans("Invalid request.");

    require MT::Entry;
    my $entry = MT::Entry->load({ id => $id, blog_id => $blog_id })
        or return $app->errtrans("Invalid request.");

    require MT::Placement;
    my @maps = MT::Placement->search( { entry_id => $entry->id } );
    my $sorter = sub {
        my ($a, $b) = @_;
        if($a->is_primary > $b->is_primary) {
            return -1;
        }
        elsif($a->is_primary < $b->is_primary) {
            return 1;
        }
        return $a->category_id <=> $b->category_id;
    };
    my $cat_ids = join ",", map { $_->category_id } sort { $sorter->($a, $b) } @maps;
    if($cat_ids) {
        $app->param('category_ids', $cat_ids);
    }
}

sub _create_temp_entry {
    my $app         = shift;

    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';

    my $type        = $app->param('_type') || 'entry';
    my $entry_class = $app->model($type);
    my $blog_id     = $app->param('blog_id');
    my $blog        = $app->blog;
    my $id          = $app->param('id');
    my $entry;
    my $user_id = $app->user->id;
    my $rn = $app->param('r');
    if(!$rn && $plugin->dynamic_preview_load_latest_revision) {
        my $rev = $plugin->latest_revision($type, $id);
        $rn = $rev->rev_number if $rev;
    }

    if ($id) {
        $entry = $entry_class->load( { id => $id, blog_id => $blog_id } )
            or return $app->errtrans("Invalid request.");
        if($rn && $blog->use_revision) {
            my $rev = $entry->load_revision({ rev_number => $rn });
            if($rev && @$rev) {
                $entry = $rev->[0];
            }
        }
        $user_id = $entry->author_id;
    }
    else {
        $entry = $entry_class->new;
        $entry->author_id($user_id);
        $entry->id(-1);    # fake out things like MT::Taggable::__load_tags
        $entry->blog_id($blog_id);
    }

    return $app->return_to_dashboard( permission => 1 )
        unless $app->permissions->can_edit_entry( $entry, $app->user );

    my $names = $entry->column_names;
    my %values = map { $_ => scalar $app->param($_) } @$names;
    delete $values{'id'} unless $app->param('id');
    ## Strip linefeed characters.
    for my $col (qw( text excerpt text_more keywords )) {
        $values{$col} =~ tr/\r//d if $values{$col};
    }
    $values{allow_comments} = 0
        if !defined( $values{allow_comments} )
        || $app->param('allow_comments') eq '';
    $values{allow_pings} = 0
        if !defined( $values{allow_pings} )
        || $app->param('allow_pings') eq '';
    $entry->set_values( \%values );

    return $entry;
}

sub preview_entry {
    my $app = shift;
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';

    $app->validate_magic or return;

    my $type = $app->param('_type') || 'entry'; 
    my $perms = $app->permissions;
    my $org_permissions = $perms->permissions || '';
    my $permissions;
    if($perms->can_do('preview')) {
        $permissions = $org_permissions;
        if($type eq 'entry' && !$perms->can_do('edit_all_posts')) {
            $permissions .= ",'edit_all_posts'";
        } 
        elsif($type eq 'page' && !$perms->can_do('manage_pages')) {
            $permissions .= ",'manage_pages'";
        }
        $perms->permissions($permissions);
    }

    require MT::CMS::Entry;
    my $tmpl = eval {
        _setup_category_ids_param($app);
        MT::CMS::Entry::preview($app, @_);
    };
    if($@) {
        trace("preview_entry: err: $@");
    }

    if($permissions) {
        $perms->permissions($org_permissions);
        $permissions = undef;
    }

    if(!$tmpl && $app->{redirect}) {
        $app->{redirect} = undef;
        return $app->errtrans('No permissions');
    }
    return $tmpl;
}

sub preview_template {
    my $app = shift;
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';

    my $perms = $app->permissions;
    my $org_permissions = $perms->permissions || '';
    my $permissions;
    if($perms->can_do('preview') && !$perms->can_do('edit_templates')) {
        $permissions = $org_permissions . ",'edit_templates'";
        $perms->permissions($permissions);
    }

    my $blog_id = $app->param('blog_id');
    MT->request('dynamic_preview_include_hold_entry',
        $plugin->dynamic_preview_include_hold_entry($blog_id));
    MT->request('dynamic_preview_include_future_entry',
        $plugin->dynamic_preview_include_future_entry($blog_id));

    my $id = $app->param('id');
    require MT::Template;
    my $tmpl = $id ? MT::Template->load($id) : undef;
    my $cat_id = $app->param('cat_id');
    my $archive_cat = $cat_id ? MT->model('category')->load($cat_id) : undef;
    no strict 'refs';
    no warnings 'redefine';
    my $saved_create_preview_content = \&MT::CMS::Template::create_preview_content;
    local *MT::CMS::Template::create_preview_content = sub {
        my ( $app, $blog, $type, $number, $cat ) = @_;
        my $join = {};
        if($cat_id) {
            $join = { 
                join => MT->model('placement')->join_on(
                    'entry_id',
                    {
                        category_id => $cat_id,
                    },
                ),
            };
        }
        my $blog_id = $blog->id;
        my $entry_class = $app->model($type);
        my @obj = $entry_class->load(
            {
                blog_id => $blog_id,
                status => MT::Entry::RELEASE(),
            },
            {
                limit => $number || 1,
                direction => 'descend',
                sort => 'authored_on',
                %$join,
            }
        );
        unless(@obj) {
            @obj = $saved_create_preview_content->($app, $blog, $type, $number, $cat); 
        }

        # HACK: archive_categoryをcms_pre_previewで差し替えるため一時的に保存する
        $app->request('dynamic_preview_archive_category', $archive_cat);

        return @obj;
    };

    require MT::CMS::Template;
    my $preview_tmpl =  eval {
        MT::CMS::Template::preview($app, @_);
    };
    if($@) {
        trace("preview_template: err: $@");
    }
    
    MT->request('dynamic_preview_include_hold_entry', undef);
    MT->request('dynamic_preview_include_future_entry', undef);

    if($permissions) {
        $perms->permissions($org_permissions);
        $permissions = undef;
    }
    
    if(!$preview_tmpl && $app->{redirect}) {
        $app->{redirect} = undef;
        return $app->errtrans('No permissions');
    }
    return $preview_tmpl;
}

sub create_role {
    my ($app) = @_;
    
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';
    if($plugin->has_extra) {
        return DynamicPreview::Extra::create_role(@_);
    }
    return $app->json_error('Not supported');
}

sub _create_temp_content_data {
    my $app         = shift;

    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';

    my $type        = $app->param('_type') || 'entry';
    my $entry_class = $app->model($type);
    my $blog_id     = $app->param('blog_id');
    my $blog        = $app->blog;
    my $id          = $app->param('id');
    my $content_type_id = $app->param('content_type_id');
    my $user_id = $app->user->id;
    my $label = $app->param('data_label');
    my $from_external_link = $app->param('from_external_link');

    return $app->errtrans('Invalid request.')
        unless $blog_id && $content_type_id;

    my $rn = $app->param('r');
    if(!$rn && $plugin->dynamic_preview_load_latest_revision) {
        my $rev = $plugin->latest_revision($type, $id);
        $rn = $rev->rev_number if $rev;
    }

    my $content_data;
    if ($id) {
        $content_data = MT::ContentData->load({
            id => $id,
            blog_id => $blog_id,
            content_type_id => $content_type_id,
        }) or return $app->errtrans("Invalid request.");
            
        if($rn && $blog->use_revision) {
            my $rev = $content_data->load_revision({ rev_number => $rn });
            if($rev && @$rev) {
                $content_data = $rev->[0];
            }
        }
        $user_id = $content_data->author_id;
    }
    else {
        $content_data = $entry_class->new;
        $content_data->set_values({
            id => -1,
            author_id => $user_id,
            blog_id => $blog_id,
            content_type_id => $content_type_id,
        });
    }

    return $app->return_to_dashboard( permission => 1 )
        unless $app->permissions->can_edit_content_data( $content_data, $app->user );

    if(!$from_external_link) {
        $content_data->status(scalar $app->param('status'));
        $content_data->label($label);
    
        my $content_field_types = $app->registry('content_field_types');
        my $content_type = $content_data->content_type;
        my $field_data = $content_type->fields;
        
        require MT::CMS::ContentData;
        my $data = {};
        foreach my $f (@$field_data) {
            my $content_field_type = $content_field_types->{$f->{type}};
            $data->{$f->{id}} = MT::CMS::ContentData::_get_form_data($app, $content_field_type, $f);
        }
        $content_data->data($data);
    }

    return $content_data;
}

sub preview_content_data {
    my $app = shift;
    my $plugin = $app->component('DynamicPreview')
        or die 'Plugin not found';

    $app->validate_magic or return;

    my $type = $app->param('_type') || 'content_data'; 
    my $perms = $app->permissions;
    my $org_permissions = $perms->permissions || '';
    my $permissions;
    if($perms->can_do('preview')) {
        $permissions = $org_permissions;
        $permissions .= ",'edit_all_content_data'";
        $perms->permissions($permissions);
    }

    require MT::CMS::ContentData;
    my $tmpl = eval {
        # _setup_category_ids_param($app);
        MT::CMS::ContentData::preview($app, @_);
    };
    if($@) {
        trace("preview_content_data: err: $@");
    }

    if($permissions) {
        $perms->permissions($org_permissions);
        $permissions = undef;
    }

    if(!$tmpl && $app->{redirect}) {
        $app->{redirect} = undef;
        return $app->errtrans('No permissions');
    }
    return $tmpl;
}


1;
__END__


